import queue.PriorityObject;

public class Vertex implements PriorityObject {
//  public static final enum Operation {INIT, EMPTY, FULL, POUR};
  public static final int INIT = 0;
  public static final int EMPTY = 1;
  public static final int FULL = 2;
  public static final int POUR = 3;
  private int oper;
  private int b1, b2;
  private int buckets[] = new int[Main.MAXBUCKETS];
  private Vertex previous = null;
  private int length = 0;
  private int value = 0;

  public Vertex(int[] init) {
    if(init.length != Main.MAXBUCKETS) {
      error();
    }
    for(int i = 0; i < Main.MAXBUCKETS; i++) {
      if((init[i] > Main.full[i]) || (init[i] < 0)) {
        error();
      }
    }
    System.arraycopy(init, 0, buckets, 0, init.length);
    oper = INIT;
  }

  private void error() {
    System.err.println("Wrong init numbers");
    System.exit(1);
  }

  public Vertex(Vertex prev, int oper, int b1, int b2) {
    previous = prev;
    this.oper = oper;
    this.length = prev.length + 1;
    System.arraycopy(prev.buckets, 0, buckets, 0, prev.buckets.length);
    this.b1 = b1;
    this.b2 = b2;
    switch(oper) {
      case EMPTY:
        buckets[b1] = 0;
        break;
      case FULL:
        buckets[b1] = Main.full[b1];
        break;
      case POUR:
        int volume = Math.min(buckets[b1], Main.full[b2] - buckets[b2]);
        buckets[b1] -= volume;
        buckets[b2] += volume;
        break;
      default:
        System.exit(1);
    }
    value = length;
    for(int i = 0; i < Main.MAXBUCKETS; i++) {
      if(buckets[i] != Main.end[i]) {
        if((Main.end[i] == 0) || (Main.end[i] == Main.full[i])) {
          value++;
        } else value += Math.abs(buckets[i] - Main.end[i]);
//        } else value += 5;
      }
    }
  }

  public int getValue() {
    return value;
  }

  public boolean isEnding() {
    for(int i = 0; i < Main.MAXBUCKETS; i++) {
      if(buckets[i] != Main.end[i]) return false;
    }
    return true;
  }

  public String toString() {
    StringBuffer str = new StringBuffer();
    str.append("Length: ");
    str.append(length);
    str.append(", Operation: ");
    switch(oper) {
      case EMPTY:
        str.append("EMPTY ");
        str.append(b1+1);
        break;
      case FULL:
        str.append("FULL ");
        str.append(b1+1);
        break;
      case POUR:
        str.append("POUR ");
        str.append(b1+1);
        str.append("->");
        str.append(b2+1);
        break;
      case INIT:
        str.append("INIT");
        break;
    }
    str.append("\n(");
    str.append(buckets[0]);
    for(int i = 1; i < Main.MAXBUCKETS; i++) {
      str.append(" ");
      str.append(buckets[i]);
    }
    str.append(")\n");
    return str.toString();
  }

  public boolean equal(Object o) {
    if(!(o instanceof Vertex)) return false;
    else {
      Vertex v = (Vertex) o;
      for(int i = 0; i < Main.MAXBUCKETS; i++) {
        if(buckets[i] != v.buckets[i]) return false;
      }
      return true;
    }
  }

  public Vertex getPrevious() {
    return previous;
  }
}
